@@ -226,9 +226,8 @@ impl Method for PasswordAuth {
                self.token_endpoint);
 
         let body = self.body.to_string().unwrap();
-        let json_type = ContentType(mime!(Application/Json));
         let resp = try!(client.post(&self.token_endpoint).body(&body)
-                        .header(json_type).send());
+                        .header(ContentType::json()).send());
         self.token_from_response(resp)
     }
 
@@ -259,7 +258,7 @@ pub mod test {
     use hyper::status::StatusCode;
 
     use super::super::super::{ApiError, ApiResult, Session};
-    use super::super::{Method, Token, SimpleToken};
+    use super::super::{Method, SimpleToken};
     use super::{Identity, PasswordAuth};
     use super::super::super::session::test;
 
@@ -381,7 +380,7 @@ pub mod test {
             .create().unwrap();
         let cli = hyper::Client::with_connector(MockToken::default());
         let token = id.get_token(&cli).unwrap();
-        assert_eq!(token.value(), "abcdef");
+        assert_eq!(&token.to_string(), "abcdef");
     }
 
     #[test]
