@@ -14,8 +14,11 @@
 
 //! Server management via Compute API.
 
+use std::collections::HashMap;
 use std::net::{Ipv4Addr, Ipv6Addr};
 
+use chrono::{DateTime, FixedOffset};
+
 use super::super::super::{ApiResult, Session, Sort};
 use super::super::super::auth::Method as AuthMethod;
 use super::super::super::service::Query;
@@ -24,7 +27,7 @@ use super::protocol;
 
 
 /// A query to server list.
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub struct ServerQuery<'a, Auth: AuthMethod + 'a> {
     service: V2ServiceWrapper<'a, Auth>,
     /// Underlying query.
@@ -72,7 +75,8 @@ pub struct ServerQuery<'a, Auth: AuthMethod + 'a> {
 /// let server = openstack::compute::v2::servers(&session)
 ///     .get("8a1c355b-2e1e-440a-8aa8-f272df72bc32")
 ///     .expect("Unable to get a server");
-/// println!("Server name is {}", server.name());
+/// println!("Server name is {}, image ID is {}, flavor ID is {}",
+///          server.name(), server.image().id(), server.flavor().id());
 /// ```
 #[derive(Debug)]
 pub struct ServerManager<'a, Auth: AuthMethod + 'a> {
@@ -96,32 +100,96 @@ pub struct ServerSummary<'a, Auth: AuthMethod + 'a> {
 /// List of servers.
 pub type ServerList<'a, Auth> = Vec<ServerSummary<'a, Auth>>;
 
+/// A reference to a flavor.
+#[derive(Debug)]
+pub struct FlavorRef<'a, Auth: AuthMethod + 'a> {
+    server: &'a Server<'a, Auth>
+}
+
+/// A reference to an image.
+#[derive(Debug)]
+pub struct ImageRef<'a, Auth: AuthMethod + 'a> {
+    server: &'a Server<'a, Auth>
+}
+
 
 impl<'a, Auth: AuthMethod + 'a> Server<'a, Auth> {
     /// Get a reference to IPv4 address.
-    pub fn access_ipv4(&self) -> &String {
+    pub fn access_ipv4(&self) -> &Option<Ipv4Addr> {
         &self.inner.accessIPv4
     }
 
     /// Get a reference to IPv6 address.
-    pub fn access_ipv6(&self) -> &String {
+    pub fn access_ipv6(&self) -> &Option<Ipv6Addr> {
         &self.inner.accessIPv6
     }
 
+    /// Get a reference to associated addresses.
+    pub fn addresses(&self) -> &HashMap<String, Vec<protocol::ServerAddress>> {
+        &self.inner.addresses
+    }
+
+    /// Get a reference to the availability zone.
+    pub fn availability_zone(&self) -> &String {
+        &self.inner.availability_zone
+    }
+
+    /// Get a reference to creation date and time.
+    pub fn created_at(&self) -> &DateTime<FixedOffset> {
+        &self.inner.created
+    }
+
+    /// Get a reference to the flavor.
+    pub fn flavor(&'a self) -> FlavorRef<'a, Auth> {
+        FlavorRef {
+            server: self
+        }
+    }
+
     /// Get a reference to server unique ID.
     pub fn id(&self) -> &String {
         &self.inner.id
     }
 
+    /// Get a reference to the image.
+    pub fn image(&'a self) -> ImageRef<'a, Auth> {
+        ImageRef {
+            server: self
+        }
+    }
+
     /// Get a reference to server name.
     pub fn name(&self) -> &String {
         &self.inner.name
     }
 
     /// Get server status.
-    pub fn status(&self) -> &String {
-        &self.inner.status
+    pub fn status(&self) -> protocol::ServerStatus {
+        self.inner.status
     }
+
+    /// Get a reference to last update date and time.
+    pub fn updated_at(&self) -> &DateTime<FixedOffset> {
+        &self.inner.updated
+    }
+}
+
+impl<'a, Auth: AuthMethod + 'a> FlavorRef<'a, Auth> {
+    /// Get a reference to flavor unique ID.
+    pub fn id(&self) -> &'a String {
+        &self.server.inner.flavor.id
+    }
+
+    // TODO: pub fn details(&self) -> ApiResult<Flavor>
+}
+
+impl<'a, Auth: AuthMethod + 'a> ImageRef<'a, Auth> {
+    /// Get a reference to image unique ID.
+    pub fn id(&self) -> &'a String {
+        &self.server.inner.image.id
+    }
+
+    // TODO: #[cfg(feature = "image")] pub fn details(&self) -> ApiResult<Image>
 }
 
 impl<'a, Auth: AuthMethod + 'a> ServerSummary<'a, Auth> {
@@ -136,8 +204,8 @@ impl<'a, Auth: AuthMethod + 'a> ServerSummary<'a, Auth> {
     }
 
     /// Get details.
-    pub fn details(self) -> ApiResult<Server<'a, Auth>> {
-        ServerManager::get_server(self.service, &self.inner.id)
+    pub fn details(&self) -> ApiResult<Server<'a, Auth>> {
+        ServerManager::get_server(self.service.clone(), &self.inner.id)
     }
 }
 
@@ -261,7 +329,7 @@ impl<'a, Auth: AuthMethod + 'a> ServerQuery<'a, Auth> {
         debug!("Received {} compute servers", inner.servers.len());
         trace!("Received servers: {:?}", inner.servers);
         Ok(inner.servers.into_iter().map(|x| ServerSummary {
-            service: service,
+            service: service.clone(),
             inner: x
         }).collect())
     }
@@ -281,7 +349,7 @@ impl<'a, Auth: AuthMethod + 'a> ServerManager<'a, Auth> {
     /// a [ServerQuery](struct.ServerQuery.html) object that
     /// you can futher specify with e.g. filtering or sorting.
     pub fn query(&self) -> ServerQuery<'a, Auth> {
-        ServerQuery::new(self.service)
+        ServerQuery::new(self.service.clone())
     }
 
     /// List all servers.
@@ -291,7 +359,7 @@ impl<'a, Auth: AuthMethod + 'a> ServerManager<'a, Auth> {
 
     /// Get a server.
     pub fn get<Id: AsRef<str>>(&self, id: Id) -> ApiResult<Server<'a, Auth>> {
-        ServerManager::get_server(self.service, id.as_ref())
+        ServerManager::get_server(self.service.clone(), id.as_ref())
     }
 
     fn get_server(service: V2ServiceWrapper<'a, Auth>, id: &str)
@@ -314,6 +382,63 @@ pub fn servers<'session, Auth>(session: &'session Session<Auth>)
     ServerManager::new(session)
 }
 
+// These implementations cannot be correctly derived
+
+impl<'a, Auth: AuthMethod + 'a> Copy for FlavorRef<'a, Auth> {}
+
+impl<'a, Auth: AuthMethod + 'a> Clone for FlavorRef<'a, Auth> {
+    fn clone(&self) -> FlavorRef<'a, Auth> {
+        FlavorRef {
+            server: self.server
+        }
+    }
+}
+
+impl<'a, Auth: AuthMethod + 'a> Copy for ImageRef<'a, Auth> {}
+
+impl<'a, Auth: AuthMethod + 'a> Clone for ImageRef<'a, Auth> {
+    fn clone(&self) -> ImageRef<'a, Auth> {
+        ImageRef {
+            server: self.server
+        }
+    }
+}
+
+impl<'a, Auth: AuthMethod + 'a> Clone for Server<'a, Auth> {
+    fn clone(&self) -> Server<'a, Auth> {
+        Server {
+            service: self.service.clone(),
+            inner: self.inner.clone()
+        }
+    }
+}
+
+impl<'a, Auth: AuthMethod + 'a> Clone for ServerManager<'a, Auth> {
+    fn clone(&self) -> ServerManager<'a, Auth> {
+        ServerManager {
+            service: self.service.clone()
+        }
+    }
+}
+
+impl<'a, Auth: AuthMethod + 'a> Clone for ServerQuery<'a, Auth> {
+    fn clone(&self) -> ServerQuery<'a, Auth> {
+        ServerQuery {
+            service: self.service.clone(),
+            query: self.query.clone()
+        }
+    }
+}
+
+impl<'a, Auth: AuthMethod + 'a> Clone for ServerSummary<'a, Auth> {
+    fn clone(&self) -> ServerSummary<'a, Auth> {
+        ServerSummary {
+            service: self.service.clone(),
+            inner: self.inner.clone()
+        }
+    }
+}
+
 
 #[cfg(test)]
 pub mod test {
