@@ -74,7 +74,8 @@ pub trait ApiVersioning {
 pub struct ServiceWrapper<'session, Auth: AuthMethod + 'session,
                           Srv: ServiceType> {
     session: &'session Session<Auth>,
-    service_type: PhantomData<Srv>
+    service_type: PhantomData<Srv>,
+    endpoint_interface: Option<String>
 }
 
 
@@ -121,7 +122,11 @@ impl<'a, Auth: AuthMethod> RequestBuilder<'a, Auth> {
     pub fn send_unchecked(self) -> ApiResult<Response> {
         let token = try!(self.parent.auth_token());
         let hdr = AuthTokenHeader(token.into());
-        self.inner.header(hdr).send().map_err(From::from)
+        self.inner.header(hdr).send().map_err(From::from).map(|resp| {
+            trace!("Got {} from {} with {:?}",
+                   resp.status, resp.url, resp.headers);
+            resp
+        })
     }
 
     /// Send this request and parse JSON response on success.
@@ -167,14 +172,25 @@ impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
             -> ServiceWrapper<'session, Auth, Srv> {
         ServiceWrapper {
             session: session,
-            service_type: PhantomData
+            service_type: PhantomData,
+            endpoint_interface: None
+        }
+    }
+
+    /// Change the endpoint interface used for this wrapper.
+    pub fn with_endpoint_interface(self, endpoint_interface: String)
+            -> ServiceWrapper<'session, Auth, Srv> {
+        ServiceWrapper {
+            endpoint_interface: Some(endpoint_interface),
+            .. self
         }
     }
 
     /// Construct and endpoint for the given service from the path.
     pub fn get_endpoint<P>(&self, path: P, query: Query) -> ApiResult<Url>
             where P: IntoIterator, P::Item: AsRef<str> {
-        let info = try!(self.session.get_service_info::<Srv>());
+        let ep = self.endpoint_interface.clone();
+        let info = try!(self.session.get_service_info::<Srv>(ep));
         let mut url = utils::url::extend(info.root_url, path);
         let _ = url.query_pairs_mut().extend_pairs(query.0);
         Ok(url)
@@ -235,15 +251,13 @@ impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
     }
 }
 
-impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
-        Copy for ServiceWrapper<'session, Auth, Srv> {}
-
 impl<'session, Auth: AuthMethod + 'session, Srv: ServiceType>
         Clone for ServiceWrapper<'session, Auth, Srv> {
     fn clone(&self) -> ServiceWrapper<'session, Auth, Srv> {
         ServiceWrapper {
             session: self.session,
-            service_type: PhantomData
+            service_type: PhantomData,
+            endpoint_interface: self.endpoint_interface.clone()
         }
     }
 }
